/* This work has been placed into the public domain. */

package kiyut.alkitab.bookviewer;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JComponent;
import javax.swing.ListSelectionModel;
import javax.swing.event.HyperlinkListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import kiyut.alkitab.api.ViewerHints;
import kiyut.alkitab.navigator.KeyListModel;
import kiyut.alkitab.options.ViewerHintsOptions;
import kiyut.alkitab.util.ComponentOrientationSupport;
import org.crosswire.jsword.book.Book;
import org.crosswire.jsword.passage.Key;
import org.crosswire.jsword.passage.PreferredKey;

/**
 * Panel which display single book with list of key on the right side
 * 
 */
public class DefinitionPane extends javax.swing.JPanel {
    
    protected BookTextPane bookTextPane;
    protected ViewerHints<ViewerHints.Key,Object> viewerHints;
    
    /** Creates new DefinitionPane */
    public DefinitionPane() {
        this(null);
    }
    
    public DefinitionPane(Book book) {
        this.viewerHints = new ViewerHints<ViewerHints.Key,Object>(ViewerHintsOptions.getInstance().getViewerHints());
        initComponents();
        initCustom();
        setBook(book);

        ComponentOrientationSupport.applyComponentOrientation(this);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jSplitPane1 = new javax.swing.JSplitPane();
        bookScrollPane = new javax.swing.JScrollPane();
        indexPane = new javax.swing.JPanel();
        indexScrollPane = new javax.swing.JScrollPane();
        indexList = new javax.swing.JList();

        setLayout(new java.awt.BorderLayout());

        jSplitPane1.setResizeWeight(1.0);
        jSplitPane1.setLeftComponent(bookScrollPane);

        indexPane.setLayout(new java.awt.BorderLayout(0, 3));

        indexScrollPane.setViewportView(indexList);

        indexPane.add(indexScrollPane, java.awt.BorderLayout.CENTER);

        jSplitPane1.setRightComponent(indexPane);

        add(jSplitPane1, java.awt.BorderLayout.CENTER);
    }// </editor-fold>//GEN-END:initComponents
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane bookScrollPane;
    protected javax.swing.JList indexList;
    protected javax.swing.JPanel indexPane;
    private javax.swing.JScrollPane indexScrollPane;
    private javax.swing.JSplitPane jSplitPane1;
    // End of variables declaration//GEN-END:variables
    
    protected void initCustom() {
        bookTextPane = new BookTextPane(viewerHints);
        bookScrollPane.setViewportView(bookTextPane);
        
        //getActionMap().setParent(bookTextPane.getActionMap());
        
        indexList.setPrototypeCellValue("DICTIONARY INDEX TEXT");
        indexList.setModel(new KeyListModel());
        indexList.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        
        indexList.addListSelectionListener(new ListSelectionListener() {
            @Override
            public void valueChanged(ListSelectionEvent evt) {
                keyValueChanged(evt);
            }
        });
    }
    
    public void setBook(Book book) {
        if (book != null) {
            List<Book> books = bookTextPane.getBooks();
            if (books.isEmpty()) {
                books.add(book);
            } else {
                books.set(0, book);
            }
            
            Key key = book.getGlobalKeyList();
            ((KeyListModel)indexList.getModel()).setKey(key);

            // for eg: DailyDevotions today key
            Key prefKey = null;
            if (book instanceof PreferredKey) {
                prefKey = ((PreferredKey)book).getPreferred();
            } 
            
            int index = 0;
            if (prefKey != null) {
                 index = key.indexOf(prefKey);
                 if (index < 0) { index = 0; }
            } 
            
            if (indexList.getModel().getSize() > index) {
                indexList.setSelectedIndex(index);
                indexList.ensureIndexIsVisible(index);
            }
        } else {
            bookTextPane.getBooks().clear();
        }
    }
    
    public Book getBook() {
        List<Book> books = bookTextPane.getBooks();
        if (books.isEmpty()) {
            return null;
        }
        return books.get(0);
    }
    
    public void setKey(Key key) {
        if (key == null) { return; }
        bookTextPane.setKey(key);
        bookTextPane.reload(true);

        // synchronize the List and the View
        int i = ((KeyListModel) indexList.getModel()).getKey().indexOf(key);

        if (i >= 0) {
            if (indexList.getSelectedIndex() != i) {
                indexList.setSelectedIndex(i);
            }
            indexList.ensureIndexIsVisible(i);
        }
    }

    /*public void setKey(keyString) {
        Book book = getBook();
        if (book == null) { return; }
        Key key = book.getValidKey(keyString);
        bookTextPane.setKey(key);
        bookTextPane.refresh(true);
    }*/
    
    public void viewSource() {
        try {
            SourceViewerPane sourcePane = new SourceViewerPane();
            sourcePane.initSource(bookTextPane);
            //sourcePane.initSource(bookTextPane.getBooks(), bookTextPane.getKey(), bookTextPane.getConverter(), bookTextPane.getViewerHints(), bookTextPane.isCompareView());
            sourcePane.showDialog(this,true);
        } catch (Exception ex) {
            Logger logger = Logger.getLogger(this.getClass().getName());
            logger.log(Level.WARNING, ex.getMessage(), ex);
        }
    }
    
    public ViewerHints<ViewerHints.Key,Object> getViewerHints() {
        if (viewerHints == null) {
            viewerHints = new ViewerHints<ViewerHints.Key,Object>();
        }
        return viewerHints;
    }

    public JComponent getViewerComponent() {
        return bookTextPane;
    }

    public void addHyperlinkListener(HyperlinkListener listener) {
        bookTextPane.addHyperlinkListener(listener);
    }
    
    public void removeHyperlinkListener(HyperlinkListener listener) {
        bookTextPane.removeHyperlinkListener(listener);
    }
    
    protected void keyValueChanged(ListSelectionEvent evt) {
        if (evt.getValueIsAdjusting()) { return; }
        int index = indexList.getSelectedIndex();
        if (index == -1) { return; }
        Key key = (Key)indexList.getModel().getElementAt(index);
        // XXX bug on JSword ?, if using string it will not find the key
        // eg: Daily Devotional
        /*System.out.println("key: " + key.toString());
        System.out.println("   : " + key.getName());
        System.out.println("   : " + key.getOsisID());
        System.out.println("   : " + key.getOsisRef());
         */
        //bookTextPane.setViewKey(key.toString());
        setKey(key);
        //System.out.println("DefinitionPane.keyValueChanged" + key);
    }
}